package com.taco.swinger.text;

import java.io.Serializable;

import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.Segment;

/** A class that provides a <code>CharSequence</code> based on a document in
 *  this resource. This class makes the assumption that the document is not
 *  changing!
 */
public class DocumentCharSequence implements CharSequence, 
  Cloneable, Serializable {
    /** Construct a new instance which encapsulates the entire document. */
    public DocumentCharSequence(Document document) {
        this(document, 0, document.getLength());
    }
    
    /** Construct a new instance starting from the argument index into the
     *  document. This is provided for efficiency purposes.
     */
    public DocumentCharSequence(Document document, int start, int end) {
        if (document == null) {
            throw new NullPointerException();
        }
        
        if ((start < 0) || (end > document.getLength())) {
            throw new IndexOutOfBoundsException();
        }
        
        _document = document;
        _start = start;
        _end = end;
    }
    
    /** Return a shallow clone of this instance. */
    public Object clone() {
        try {   
            return super.clone();
        } catch (CloneNotSupportedException cnse) {
            throw new InternalError();
        }            
    }
    
    public char charAt(int index) {
        try {
            return _document.getText(_start + index, 1).charAt(0);
        } catch (BadLocationException e) {
            throw new RuntimeException("document.getText() failed", e);
        }
    }
            
    public int length() { return (_end - _start); }
                                     
    public CharSequence subSequence(int start, int end) {
        int length = _end - _start;
        int targetLength = end - start;
        
        if ((start < 0) || (start > end) ||
            (targetLength > length)) {
            throw new IndexOutOfBoundsException("start = " + start +
            ", end = " + end);
        }    
                
        // For efficiency purposes, instead of creating a string 
        // corresponding to the subsequence, return a new instance
        // of DocumentCharSequence that is offset by the appropriate
        // amount.            
        return new DocumentCharSequence(_document, _start + start, 
          _start + end); 
    }
        
    public String toString() {
        try {
            return _document.getText(_start, _end - _start);
        } catch (BadLocationException e) {
            throw new RuntimeException("Document.getText() failed", e);
        }                
    }                 
    
    /** The document that this instance is based on. */
    protected final Document _document;
    
    /** The index into the document that corresponds to the first 
     *  character in this sequence.
     */
    protected final int _start;
    
    /** The index into the document that corresponds to just after the 
     *  last character in this sequence.
     */
    protected final int _end;
};
